==> tmap_map_fluttify-0.0.5.tar.gz: lib/src/facade/models.dart <==
import 'package:core_location_fluttify/core_location_fluttify.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:latlng/latlng.dart';
import 'package:tmap_map_fluttify/src/android/android.export.g.dart';
import 'package:tmap_map_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';

@immutable
class MyLocationOption {
  MyLocationOption({
    this.show = true,
    this.myLocationType = MyLocationType.Locate,
    this.iconUri,
    this.package,
    this.imageConfiguration,
    this.strokeColor,
    this.strokeWidth,
    this.fillColor,
  }) : assert(
          (iconUri != null && imageConfiguration != null) || iconUri == null,
          'iconUri与imageConfiguration同时设置!',
        );

  /// 是否显示
  final bool show;

  /// 定位类型
  final MyLocationType myLocationType;

  /// 我的位置图标
  final Uri iconUri;

  /// 图片所在package, 通AssetImage构造器里的package参数
  final String package;

  /// 图标配置
  final ImageConfiguration imageConfiguration;

  /// 边框颜色
  final Color strokeColor;

  /// 边框宽度
  final double strokeWidth;

  /// 填充颜色
  final Color fillColor;

  @override
  String toString() {
    return 'MyLocationOption{show: $show}';
  }
}

@immutable
class MarkerOption {
  /// 经纬度
  final LatLng latLng;

  /// 标题
  final String title;

  /// 副标题
  final String snippet;

  /// 图片uri 可以是url, asset路径或者文件路径
  ///
  /// 如果设置了[iconUri], 那么必须同时设置[imageConfig], 否则图片大小会不一致, 这是flutter
  /// 的bug
  final Uri iconUri;

  /// 图片参数
  ///
  /// 目前利用到的信息只有[devicePixelRatio], 使用[devicePixelRatio]获取当前设备
  /// 对应分辨率的图片(Android), iOS使用1.0x的图片. 所以[size]设置了是没用的, 这是flutter
  /// 的PlatformView的bug, 参考https://github.com/flutter/flutter/issues/24865.
  /// 这个bug彻底解决之后才能保证marker是完美的.
  final ImageConfiguration imageConfig;

  /// 是否可拖动
  final bool draggable;

  /// 是否允许弹窗
  final bool infoWindowEnabled;

  /// 是否可见
  final bool visible;

  /// 横轴锚点
  final double anchorU;

  /// 纵轴锚点
  final double anchorV;

  /// 自定义数据 理论上可以使用任何类型的数据, 但是为了减少意外情况, 这里一律转换成String来保存
  final String object;

  MarkerOption({
    @required this.latLng,
    this.title = '',
    this.snippet = '',
    this.iconUri,
    this.imageConfig,
    this.draggable = false,
    this.infoWindowEnabled = true,
    this.visible = true,
    this.anchorU = 0.5,
    this.anchorV = 0,
    this.object,
  }) : assert(
          (iconUri != null && imageConfig != null) || iconUri == null,
          'iconUri和imageConfig必须同时设置! 如果想要一个默认的imageConfig, 那么就直接调用[createLocalImageConfiguration]方法来创建!',
        );

  @override
  String toString() {
    return 'MarkerOption{latLng: $latLng, title: $title, snippet: $snippet, iconUri: $iconUri, imageConfig: $imageConfig, draggable: $draggable, infoWindowEnabled: $infoWindowEnabled, visible: $visible, anchorU: $anchorU, anchorV: $anchorV, object: $object}';
  }
}

class Marker {
  Marker.android(this.androidModel);

  Marker.ios(this.iosModel, this._annotationView, this.iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Marker androidModel;

  QPointAnnotation iosModel;
  QAnnotationView _annotationView;
  QMapView iosController;

  /// 获取标题
  Future<String> get title {
    return platform(
      android: (_) => androidModel.getTitle(),
      ios: (_) => iosModel.get_title(),
    );
  }

  /// 获取副标题
  Future<String> get snippet {
    return platform(
      android: (_) => androidModel.getSnippet(),
      ios: (_) => iosModel.get_subtitle(),
    );
  }

  /// 获取定位信息
  Future<LatLng> get location {
    return platform(
      android: (_) async {
        final _location = await androidModel.getPosition();
        return LatLng(
          await _location.get_latitude(),
          await _location.get_longitude(),
        );
      },
      ios: (_) async {
        final location = await iosModel.get_coordinate();
        return LatLng(await location.latitude, await location.longitude);
      },
    );
  }

  /// 获取自定义信息
  Future<String> get object {
    return platform(
      android: (_) => androidModel.getTitle(),
      ios: (_) async {
        final result = await iosModel.getJsonableProperty__(7);
        return result as String;
      },
    );
  }

  /// 删除marker
  Future<void> remove() async {
    return platform(
      android: (_) => androidModel.remove(),
      ios: (_) => iosController.removeAnnotation(iosModel),
    );
  }

  /// 设置坐标
  Future<void> setCoordinate(LatLng coord) async {
    assert(coord != null);
    return platform(
      android: (_) async => androidModel.setPosition(
        await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(
          coord.latitude,
          coord.longitude,
        ),
      ),
      ios: (_) async {
        if (_annotationView != null) {
          final coordinate = await CLLocationCoordinate2D.create(
            coord.latitude,
            coord.longitude,
          );
          await iosModel.set_coordinate(coordinate);
          await _annotationView.set_annotation(iosModel, viewChannel: false);
        } else {
          debugPrint('当前_annotationView为null, 无法设置经纬度!');
        }
      },
    );
  }

  /// 设置可见性
  Future<void> setVisible(bool visible) async {
    assert(visible != null);
    return platform(
      android: (_) => androidModel.setVisible(visible),
      ios: (_) async {
        debugPrint('ios端目前无法设置可见性!');
        if (_annotationView != null) {
          await _annotationView.setHidden(!visible);
        } else {
          debugPrint('当前_annotationView为null, 无法设置可见性!');
        }
      },
    );
  }

  /// 显示弹窗
  Future<void> showInfoWindow() async {
    return platform(
      android: (_) async {
        androidModel.showInfoWindow();
      },
      ios: (_) async {
        iosController?.selectAnnotation_animated(iosModel, true);
      },
    );
  }

  /// 关闭弹窗
  Future<void> hideInfoWindow() async {
    return platform(
      android: (_) => androidModel.hideInfoWindow(),
      ios: (_) => iosController?.deselectAnnotation_animated(iosModel, true),
    );
  }

  /// 设置图标
  Future<void> setIcon(Uri iconUri, ImageConfiguration configuration) async {
    return platform(
      android: (pool) async {
        final iconData = await uri2ImageData(configuration, iconUri);

        final bitmap = await android_graphics_Bitmap.create(iconData);
        final icon =
            await com_tencent_tencentmap_mapsdk_maps_model_BitmapDescriptorFactory
                .fromBitmap__android_graphics_Bitmap(bitmap);
        await androidModel.setIcon(icon);
      },
      ios: (pool) async {
        final iconData = await uri2ImageData(configuration, iconUri);

        final icon = await UIImage.create(iconData);

        _annotationView.set_image(icon, viewChannel: false);
      },
    );
  }
}

/// 屏幕坐标
@immutable
class Point {
  final double x;
  final double y;

  Point(this.x, this.y);

  @override
  String toString() {
    return 'Point{x: $x, y: $y}';
  }
}

/// 屏幕坐标
@immutable
class TraceLocation {
  final double latitude;
  final double longitude;
  final double speed;
  final double bearing;
  final int time;

  TraceLocation({
    @required this.latitude,
    @required this.longitude,
    @required this.speed,
    @required this.bearing,
    @required this.time,
  });

  @override
  String toString() {
    return 'TraceLocation{latitude: $latitude, longitude: $longitude, speed: $speed, bearing: $bearing, time: $time}';
  }
}

/// 海量点创建参数
@immutable
class MultiPointOption {
  /// 点列表
  final List<PointOption> pointList;

  /// 图片uri
  final Uri iconUri;

  /// 图片配置
  final ImageConfiguration imageConfiguration;

  /// 图片大小 仅限ios
  final Size size;

  MultiPointOption({
    @required this.pointList,
    this.iconUri,
    this.imageConfiguration,
    this.size,
  });

  @override
  String toString() {
    return 'MultiPointOption{pointList: $pointList, iconUri: $iconUri, imageConfiguration: $imageConfiguration, size: $size}';
  }
}

/// 海量点中单个点的创建参数
@immutable
class PointOption {
  /// 经纬度
  final LatLng latLng;

  /// 点的id列表, 用来区分点
  final String id;

  /// 标题列表
  final String title;

  /// 副标题列表
  final String snippet;

  /// 自定义数据
  final String object;

  PointOption({
    @required this.latLng,
    this.id,
    this.title,
    this.snippet,
    this.object,
  });

  @override
  String toString() {
    return 'PointOption{latLng: $latLng, id: $id, title: $title, snippet: $snippet, object: $object}';
  }
}

/// 地图移动
@immutable
class MapMove {
  final LatLng latLng;
  final double zoom;
  final double tilt;
  final bool isAbroad;

  MapMove({this.latLng, this.zoom, this.tilt, this.isAbroad});

  @override
  String toString() {
    return 'MapDrag{latLng: $latLng, zoom: $zoom, tilt: $tilt, isAbroad: $isAbroad}';
  }
}

/// Polyline创建参数
@immutable
class PolylineOption {
  /// 经纬度列表
  final List<LatLng> latLngList;

  /// 宽度
  final double width;

  /// 颜色
  final Color strokeColor;

  /// 自定义纹理
  final Uri customTexture;

  /// 图片参数
  final ImageConfiguration imageConfig;

  /// 线段末端样式
  final LineCapType lineCapType;

  /// 线段连接处样式
  final LineJoinType lineJoinType;

  /// 是否虚线
  final DashType dashType;

  PolylineOption({
    @required this.latLngList,
    this.width = 5,
    this.strokeColor = Colors.green,
    this.customTexture,
    this.imageConfig,
    this.lineCapType,
    this.lineJoinType,
    this.dashType,
  }) : assert(
          (customTexture != null && imageConfig != null) ||
              customTexture == null,
          'customTexture和imageConfig必须同时设置! 如果想要一个默认的imageConfig, 那么就直接调用[createLocalImageConfiguration]方法来创建!',
        );

  @override
  String toString() {
    return 'PolylineOption{latLngList: $latLngList, width: $width, strokeColor: $strokeColor, customTexture: $customTexture, imageConfig: $imageConfig, lineCapType: $lineCapType, lineJoinType: $lineJoinType, dotted: $dashType}';
  }
}

/// Polygon创建参数
@immutable
class PolygonOption {
  /// 经纬度列表
  final List<LatLng> latLngList;

  /// 宽度
  final double width;

  /// 边框颜色
  final Color strokeColor;

  /// 填充颜色
  final Color fillColor;

  PolygonOption({
    @required this.latLngList,
    this.width = 5,
    this.strokeColor = Colors.green,
    this.fillColor = Colors.transparent,
  });

  @override
  String toString() {
    return 'PolygonOption{latLngList: $latLngList, width: $width, strokeColor: $strokeColor, fillColor: $fillColor}';
  }
}

/// Circle创建参数
@immutable
class CircleOption {
  /// 中心点经纬度
  final LatLng center;

  /// 宽度
  final double radius;

  /// 宽度
  final double width;

  /// 边框颜色
  final Color strokeColor;

  /// 填充颜色
  final Color fillColor;

  CircleOption({
    @required this.center,
    @required this.radius,
    this.width = 5,
    this.strokeColor = Colors.green,
    this.fillColor = Colors.transparent,
  })  : assert(center != null),
        assert(radius != null);

  @override
  String toString() {
    return 'CircleOption{center: $center, radius: $radius, width: $width, strokeColor: $strokeColor, fillColor: $fillColor}';
  }
}

/// TileOverlay创建参数
@immutable
class HeatmapTileOption {
  /// 中心点经纬度
  final List<LatLng> latLngList;

  HeatmapTileOption({
    @required this.latLngList,
  });

  @override
  String toString() {
    return 'HeatmapTileOption{latLngList: $latLngList}';
  }
}

/// 图片覆盖物创建参数
@immutable
class GroundOverlayOption {
  final LatLng southWest;
  final LatLng northEast;
  final Uri imageUri;
  final ImageConfiguration imageConfiguration;

  GroundOverlayOption({
    @required this.southWest,
    @required this.northEast,
    @required this.imageUri,
    @required this.imageConfiguration,
  }) : assert(
          (imageUri != null && imageConfiguration != null) || imageUri == null,
          'imageUri与imageConfiguration同时设置!',
        );

  @override
  String toString() {
    return 'GroundOverlayOption{southWest: $southWest, northEast: $northEast, imageUri: $imageUri, imageConfiguration: $imageConfiguration}';
  }
}

/// 平滑移动Marker创建参数
@immutable
class SmoothMoveMarkerOption {
  /// 轨迹经纬度列表
  final List<LatLng> path;

  /// 图片uri 可以是url, asset路径或者文件路径
  ///
  /// 如果设置了[iconUri], 那么必须同时设置[imageConfig], 否则图片大小会不一致, 这是flutter
  /// 的bug
  final Uri iconUri;

  /// 图片参数
  ///
  /// 目前利用到的信息只有[devicePixelRatio], 使用[devicePixelRatio]获取当前设备
  /// 对应分辨率的图片(Android), iOS使用1.0x的图片. 所以[size]设置了是没用的, 这是flutter
  /// 的PlatformView的bug, 参考https://github.com/flutter/flutter/issues/24865.
  /// 这个bug彻底解决之后才能保证marker是完美的.
  final ImageConfiguration imageConfig;

  /// 动画时长
  final Duration duration;

  SmoothMoveMarkerOption({
    @required this.path,
    @required this.iconUri,
    @required this.imageConfig,
    @required this.duration,
  }) : assert(
          (iconUri != null && imageConfig != null) || iconUri == null,
          'iconUri和imageConfig必须同时设置! 如果想要一个默认的imageConfig, 那么就直接调用[createLocalImageConfiguration]方法来创建!',
        );

  @override
  String toString() {
    return 'SmoothMoveMarkerOption{path: $path, iconUri: $iconUri, imageConfig: $imageConfig, duration: $duration}';
  }
}

/// 折线
class Polyline {
  Polyline.android(this._androidModel);

  Polyline.ios(this._iosModel, this._iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Polyline _androidModel;

  QPolyline _iosModel;
  QMapView _iosController;

  Future<void> remove() {
    return platform(
      android: (_) => _androidModel.remove(),
      ios: (_) => _iosController?.removeOverlay(_iosModel),
    );
  }
}

/// 多边形
class Polygon {
  Polygon.android(this._androidModel);

  Polygon.ios(this._iosModel, this._iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Polygon _androidModel;
  QPolygon _iosModel;
  QMapView _iosController;

  Future<void> remove() {
    return platform(
      android: (_) => _androidModel.remove(),
      ios: (_) => _iosController?.removeOverlay(_iosModel),
    );
  }

  Future<bool> contains(LatLng target) {
    return platform(
      android: (_) async {
        final latLng = await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(target.latitude, target.longitude);
        return _androidModel.contains(latLng);
      },
      ios: (_) async {
        final latLng = await CLLocationCoordinate2D.create(
            target.latitude, target.longitude);
        final point = await QMapPointForCoordinate(latLng);
        final bounds = await _iosModel.get_points();
        return QPolygonContainsPoint(point, bounds, bounds.length);
      },
    );
  }
}

/// 圆形
class Circle {
  Circle.android(this._androidModel);

  Circle.ios(this._iosModel, this._iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Circle _androidModel;
  QCircle _iosModel;
  QMapView _iosController;

  Future<void> remove() {
    return platform(
      android: (_) => _androidModel.remove(),
      ios: (_) => _iosController?.removeOverlay(_iosModel),
    );
  }
}

==> tmap_map_fluttify-0.2.0.tar.gz: lib/src/facade/models.dart <==
import 'package:core_location_fluttify/core_location_fluttify.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tmap_map_fluttify/src/android/android.export.g.dart';
import 'package:tmap_map_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';
import 'extensions.dart';

@immutable
class MyLocationOption {
  MyLocationOption({
    this.show = true,
    this.myLocationType = MyLocationType.Locate,
    this.iconProvider,
    this.strokeColor,
    this.strokeWidth,
    this.fillColor,
  });

  /// 是否显示
  final bool show;

  /// 定位类型
  final MyLocationType myLocationType;

  /// 图标
  ///
  /// 资源图片则使用[AssetImage], 网络图片则使用[NetworkImage], 文件图片则使用[FileImage]
  final ImageProvider iconProvider;

  /// 边框颜色
  final Color strokeColor;

  /// 边框宽度
  final double strokeWidth;

  /// 填充颜色
  final Color fillColor;

  @override
  String toString() {
    return 'MyLocationOption{show: $show, myLocationType: $myLocationType, iconProvider: $iconProvider, strokeColor: $strokeColor, strokeWidth: $strokeWidth, fillColor: $fillColor}';
  }
}

@immutable
class MarkerOption {
  /// 经纬度
  final LatLng position;

  /// 标题
  final String title;

  /// 副标题
  final String snippet;

  /// 图标
  final ImageProvider iconProvider;

  /// 是否可拖动
  final bool draggable;

  /// 是否允许弹窗
  final bool infoWindowEnabled;

  /// 是否可见
  final bool visible;

  /// 横轴锚点
  final double anchorU;

  /// 纵轴锚点
  final double anchorV;

  /// 自定义数据 理论上可以使用任何类型的数据, 但是为了减少意外情况, 这里一律转换成String来保存
  final String object;

  MarkerOption({
    @required this.position,
    this.title = '',
    this.snippet = '',
    this.iconProvider,
    this.draggable = false,
    this.infoWindowEnabled = true,
    this.visible = true,
    this.anchorU = 0.5,
    this.anchorV = 0,
    this.object,
  });

  @override
  String toString() {
    return 'MarkerOption{latLng: $position, title: $title, snippet: $snippet, iconProvider: $iconProvider, draggable: $draggable, infoWindowEnabled: $infoWindowEnabled, visible: $visible, anchorU: $anchorU, anchorV: $anchorV, object: $object}';
  }
}

/// 屏幕坐标
@immutable
class Point {
  final double x;
  final double y;

  Point(this.x, this.y);

  @override
  String toString() {
    return 'Point{x: $x, y: $y}';
  }
}

/// 屏幕坐标
@immutable
class TraceLocation {
  final double latitude;
  final double longitude;
  final double speed;
  final double bearing;
  final int time;

  TraceLocation({
    @required this.latitude,
    @required this.longitude,
    @required this.speed,
    @required this.bearing,
    @required this.time,
  });

  @override
  String toString() {
    return 'TraceLocation{latitude: $latitude, longitude: $longitude, speed: $speed, bearing: $bearing, time: $time}';
  }
}

/// 海量点创建参数
@immutable
class MultiPointOption {
  /// 点列表
  final List<PointOption> pointList;

  /// 图片uri
  final Uri iconUri;

  /// 图片配置
  final ImageConfiguration imageConfiguration;

  /// 图片大小 仅限ios
  final Size size;

  MultiPointOption({
    @required this.pointList,
    this.iconUri,
    this.imageConfiguration,
    this.size,
  });

  @override
  String toString() {
    return 'MultiPointOption{pointList: $pointList, iconUri: $iconUri, imageConfiguration: $imageConfiguration, size: $size}';
  }
}

/// 海量点中单个点的创建参数
@immutable
class PointOption {
  /// 经纬度
  final LatLng latLng;

  /// 点的id列表, 用来区分点
  final String id;

  /// 标题列表
  final String title;

  /// 副标题列表
  final String snippet;

  /// 自定义数据
  final String object;

  PointOption({
    @required this.latLng,
    this.id,
    this.title,
    this.snippet,
    this.object,
  });

  @override
  String toString() {
    return 'PointOption{latLng: $latLng, id: $id, title: $title, snippet: $snippet, object: $object}';
  }
}

/// 地图移动
@immutable
class MapMove {
  final LatLng latLng;
  final double zoom;
  final double tilt;
  final bool isAbroad;

  MapMove({this.latLng, this.zoom, this.tilt, this.isAbroad});

  @override
  String toString() {
    return 'MapDrag{latLng: $latLng, zoom: $zoom, tilt: $tilt, isAbroad: $isAbroad}';
  }
}

/// Polyline创建参数
@immutable
class PolylineOption {
  /// 经纬度列表
  final List<LatLng> latLngList;

  /// 宽度
  final double width;

  /// 颜色
  final Color strokeColor;

  /// 自定义纹理
  final ImageProvider customTexture;

  /// 图片参数
  final ImageConfiguration imageConfig;

  /// 线段末端样式
  final LineCapType lineCapType;

  /// 线段连接处样式
  final LineJoinType lineJoinType;

  /// 是否虚线
  final DashType dashType;

  PolylineOption({
    @required this.latLngList,
    this.width = 5,
    this.strokeColor = Colors.green,
    this.customTexture,
    this.imageConfig,
    this.lineCapType,
    this.lineJoinType,
    this.dashType,
  }) : assert(
          (customTexture != null && imageConfig != null) ||
              customTexture == null,
          'customTexture和imageConfig必须同时设置! 如果想要一个默认的imageConfig, 那么就直接调用[createLocalImageConfiguration]方法来创建!',
        );

  @override
  String toString() {
    return 'PolylineOption{latLngList: $latLngList, width: $width, strokeColor: $strokeColor, customTexture: $customTexture, imageConfig: $imageConfig, lineCapType: $lineCapType, lineJoinType: $lineJoinType, dotted: $dashType}';
  }
}

/// Polygon创建参数
@immutable
class PolygonOption {
  /// 经纬度列表
  final List<LatLng> latLngList;

  /// 宽度
  final double width;

  /// 边框颜色
  final Color strokeColor;

  /// 填充颜色
  final Color fillColor;

  PolygonOption({
    @required this.latLngList,
    this.width = 5,
    this.strokeColor = Colors.green,
    this.fillColor = Colors.transparent,
  });

  @override
  String toString() {
    return 'PolygonOption{latLngList: $latLngList, width: $width, strokeColor: $strokeColor, fillColor: $fillColor}';
  }
}

/// Circle创建参数
@immutable
class CircleOption {
  /// 中心点经纬度
  final LatLng center;

  /// 宽度
  final double radius;

  /// 宽度
  final double width;

  /// 边框颜色
  final Color strokeColor;

  /// 填充颜色
  final Color fillColor;

  CircleOption({
    @required this.center,
    @required this.radius,
    this.width = 5,
    this.strokeColor = Colors.green,
    this.fillColor = Colors.transparent,
  })  : assert(center != null),
        assert(radius != null);

  @override
  String toString() {
    return 'CircleOption{center: $center, radius: $radius, width: $width, strokeColor: $strokeColor, fillColor: $fillColor}';
  }
}

/// TileOverlay创建参数
@immutable
class HeatmapTileOption {
  /// 中心点经纬度
  final List<LatLng> latLngList;

  HeatmapTileOption({
    @required this.latLngList,
  });

  @override
  String toString() {
    return 'HeatmapTileOption{latLngList: $latLngList}';
  }
}

/// 图片覆盖物创建参数
@immutable
class GroundOverlayOption {
  final LatLng southWest;
  final LatLng northEast;
  final Uri imageUri;
  final ImageConfiguration imageConfiguration;

  GroundOverlayOption({
    @required this.southWest,
    @required this.northEast,
    @required this.imageUri,
    @required this.imageConfiguration,
  }) : assert(
          (imageUri != null && imageConfiguration != null) || imageUri == null,
          'imageUri与imageConfiguration同时设置!',
        );

  @override
  String toString() {
    return 'GroundOverlayOption{southWest: $southWest, northEast: $northEast, imageUri: $imageUri, imageConfiguration: $imageConfiguration}';
  }
}

/// 平滑移动Marker创建参数
@immutable
class SmoothMoveMarkerOption {
  /// 轨迹经纬度列表
  final List<LatLng> path;

  /// 图片uri 可以是url, asset路径或者文件路径
  ///
  /// 如果设置了[iconUri], 那么必须同时设置[imageConfig], 否则图片大小会不一致, 这是flutter
  /// 的bug
  final Uri iconUri;

  /// 图片参数
  ///
  /// 目前利用到的信息只有[devicePixelRatio], 使用[devicePixelRatio]获取当前设备
  /// 对应分辨率的图片(Android), iOS使用1.0x的图片. 所以[size]设置了是没用的, 这是flutter
  /// 的PlatformView的bug, 参考https://github.com/flutter/flutter/issues/24865.
  /// 这个bug彻底解决之后才能保证marker是完美的.
  final ImageConfiguration imageConfig;

  /// 动画时长
  final Duration duration;

  SmoothMoveMarkerOption({
    @required this.path,
    @required this.iconUri,
    @required this.imageConfig,
    @required this.duration,
  }) : assert(
          (iconUri != null && imageConfig != null) || iconUri == null,
          'iconUri和imageConfig必须同时设置! 如果想要一个默认的imageConfig, 那么就直接调用[createLocalImageConfiguration]方法来创建!',
        );

  @override
  String toString() {
    return 'SmoothMoveMarkerOption{path: $path, iconUri: $iconUri, imageConfig: $imageConfig, duration: $duration}';
  }
}

class Marker {
  Marker.android(this.androidModel);

  Marker.ios(this.iosModel, this.iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Marker androidModel;

  QPointAnnotation iosModel;
  QMapView iosController;

  /// 获取标题
  Future<String> get title {
    return platform(
      android: (_) => androidModel.getTitle(),
      ios: (_) => iosModel.get_title(),
    );
  }

  /// 获取副标题
  Future<String> get snippet {
    return platform(
      android: (_) => androidModel.getSnippet(),
      ios: (_) => iosModel.get_subtitle(),
    );
  }

  /// 获取定位信息
  Future<LatLng> get position {
    return platform(
      android: (_) async {
        final _location = await androidModel.getPosition();
        return LatLng(
          await _location.get_latitude(),
          await _location.get_longitude(),
        );
      },
      ios: (_) async {
        final location = await iosModel.get_coordinate();
        return LatLng(await location.latitude, await location.longitude);
      },
    );
  }

  /// 获取自定义信息
  Future<String> get object {
    return platform(
      android: (_) {
        return androidModel.getTag().then((object) => object as String);
      },
      ios: (_) {
        return iosModel
            .getJsonableProperty__(7)
            .then((object) => object as String);
      },
    );
  }

  /// 删除marker
  Future<void> remove() async {
    return platform(
      android: (_) => androidModel.remove(),
      ios: (_) => iosController.removeAnnotation(iosModel),
    );
  }

  /// 设置坐标
  Future<void> setCoordinate(LatLng coord) async {
    assert(coord != null);
    return platform(
      android: (_) async => androidModel.setPosition(
        await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(
          coord.latitude,
          coord.longitude,
        ),
      ),
      ios: (_) async {
        final annotationView = await iosController.viewForAnnotation(iosModel);
        if (annotationView != null) {
          final coordinate = await CLLocationCoordinate2D.create(
            coord.latitude,
            coord.longitude,
          );
          await iosModel.set_coordinate(coordinate);
          await annotationView.set_annotation(iosModel, viewChannel: false);
        } else {
          debugPrint('当前_annotationView为null, 无法设置经纬度!');
        }
      },
    );
  }

  /// 设置可见性
  Future<void> setVisible(bool visible) async {
    assert(visible != null);
    return platform(
      android: (_) => androidModel.setVisible(visible),
      ios: (_) async {
        await iosModel.setVisible(visible);

        final annotationView = await iosController.viewForAnnotation(iosModel);
        await annotationView?.setHidden(!visible);
      },
    );
  }

  /// 显示弹窗
  Future<void> showInfoWindow() async {
    return platform(
      android: (_) => androidModel.showInfoWindow(),
      ios: (_) => iosController.selectAnnotation_animated(iosModel, true),
    );
  }

  /// 关闭弹窗
  Future<void> hideInfoWindow() async {
    return platform(
      android: (_) => androidModel.hideInfoWindow(),
      ios: (_) => iosController?.deselectAnnotation_animated(iosModel, true),
    );
  }

  /// 设置图标
  Future<void> setIcon(
    ImageProvider iconProvider,
    ImageConfiguration configuration,
  ) async {
    final iconData = await iconProvider.toImageData(configuration);
    return platform(
      android: (pool) async {
        final bitmap = await android_graphics_Bitmap.create(iconData);
        final icon =
            await com_tencent_tencentmap_mapsdk_maps_model_BitmapDescriptorFactory
                .fromBitmap__android_graphics_Bitmap(bitmap);
        await androidModel.setIcon(icon);
      },
      ios: (pool) async {
        final icon = await UIImage.create(iconData);

        // 这里和annotationView?.set_image看上去是做同一件事情, 但其实是针对两种不同情况
        // 如果marker是屏幕内, 那会直接走annotationView?.set_image;
        // 如果不在屏幕内, 那么annotationView?.set_image不会被执行, 如果此marker后来
        // 进入到屏幕内后, 此时需要同步annotation的数据, 如果不给annotation设置值, 那么
        // 渲染的时候还是上次的图片
        await iosModel.setIcon(icon);
        final annotationView = await iosController.viewForAnnotation(iosModel);
        await annotationView?.set_image(icon, viewChannel: false);
      },
    );
  }
}

/// 折线
class Polyline {
  Polyline.android(this._androidModel);

  Polyline.ios(this._iosModel, this._iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Polyline _androidModel;

  QPolyline _iosModel;
  QMapView _iosController;

  Future<void> remove() {
    return platform(
      android: (_) => _androidModel.remove(),
      ios: (_) => _iosController?.removeOverlay(_iosModel),
    );
  }
}

/// 多边形
class Polygon {
  Polygon.android(this._androidModel);

  Polygon.ios(this._iosModel, this._iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Polygon _androidModel;
  QPolygon _iosModel;
  QMapView _iosController;

  Future<void> remove() {
    return platform(
      android: (_) => _androidModel.remove(),
      ios: (_) => _iosController?.removeOverlay(_iosModel),
    );
  }

  Future<bool> contains(LatLng target) {
    return platform(
      android: (_) async {
        final latLng = await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(target.latitude, target.longitude);
        return _androidModel.contains(latLng);
      },
      ios: (_) async {
        final latLng = await CLLocationCoordinate2D.create(
            target.latitude, target.longitude);
        final point = await QMapPointForCoordinate(latLng);
        final bounds = await _iosModel.get_points();
        return QPolygonContainsPoint(point, bounds, bounds.length);
      },
    );
  }
}

/// 圆形
class Circle {
  Circle.android(this._androidModel);

  Circle.ios(this._iosModel, this._iosController);

  com_tencent_tencentmap_mapsdk_maps_model_Circle _androidModel;
  QCircle _iosModel;
  QMapView _iosController;

  Future<void> remove() {
    return platform(
      android: (_) => _androidModel.remove(),
      ios: (_) => _iosController?.removeOverlay(_iosModel),
    );
  }
}
